@@ -51,6 +51,8 @@ struct _grn_ii {
 #define GRN_II_MAX_CHUNK          (1 << (GRN_II_W_TOTAL_CHUNK - GRN_II_W_CHUNK))
 #define GRN_II_N_CHUNK_VARIATION  (GRN_II_W_CHUNK - GRN_II_W_LEAST_CHUNK)
 
+#define GRN_II_MAX_CHUNK_TINY     (1 << (GRN_II_W_TOTAL_CHUNK - GRN_II_W_CHUNK - 8))
+
 struct grn_ii_header {
   uint64_t total_chunk_size;
   uint64_t bmax;
@@ -99,6 +101,7 @@ GRN_API grn_ii *grn_ii_open(grn_ctx *ctx, const char *path, grn_obj *lexicon);
 GRN_API grn_rc grn_ii_close(grn_ctx *ctx, grn_ii *ii);
 GRN_API grn_rc grn_ii_remove(grn_ctx *ctx, const char *path);
 grn_rc grn_ii_info(grn_ctx *ctx, grn_ii *ii, uint64_t *seg_size, uint64_t *chunk_size);
+grn_column_flags grn_ii_get_flags(grn_ctx *ctx, grn_ii *ii);
 grn_rc grn_ii_update_one(grn_ctx *ctx, grn_ii *ii, uint32_t key, grn_ii_updspec *u,
                          grn_hash *h);
 grn_rc grn_ii_delete_one(grn_ctx *ctx, grn_ii *ii, uint32_t key, grn_ii_updspec *u,
