@@ -32,7 +32,7 @@ extern "C" {
 void grn_proc_init_from_env(void);
 
 GRN_VAR const char *grn_document_root;
-void grn_db_init_builtin_query(grn_ctx *ctx);
+void grn_db_init_builtin_commands(grn_ctx *ctx);
 
 void grn_proc_init_clearlock(grn_ctx *ctx);
 void grn_proc_init_column_copy(grn_ctx *ctx);
@@ -44,6 +44,7 @@ void grn_proc_init_config_get(grn_ctx *ctx);
 void grn_proc_init_config_set(grn_ctx *ctx);
 void grn_proc_init_config_delete(grn_ctx *ctx);
 void grn_proc_init_define_selector(grn_ctx *ctx);
+void grn_proc_init_dump(grn_ctx *ctx);
 void grn_proc_init_edit_distance(grn_ctx *ctx);
 void grn_proc_init_fuzzy_search(grn_ctx *ctx);
 void grn_proc_init_highlight(grn_ctx *ctx);
@@ -54,7 +55,9 @@ void grn_proc_init_lock_clear(grn_ctx *ctx);
 void grn_proc_init_lock_release(grn_ctx *ctx);
 void grn_proc_init_object_exist(grn_ctx *ctx);
 void grn_proc_init_object_inspect(grn_ctx *ctx);
+void grn_proc_init_object_list(grn_ctx *ctx);
 void grn_proc_init_object_remove(grn_ctx *ctx);
+void grn_proc_init_query_expand(grn_ctx *ctx);
 void grn_proc_init_schema(grn_ctx *ctx);
 void grn_proc_init_select(grn_ctx *ctx);
 void grn_proc_init_snippet(grn_ctx *ctx);
@@ -63,6 +66,8 @@ void grn_proc_init_table_create(grn_ctx *ctx);
 void grn_proc_init_table_list(grn_ctx *ctx);
 void grn_proc_init_table_remove(grn_ctx *ctx);
 void grn_proc_init_table_rename(grn_ctx *ctx);
+void grn_proc_init_table_tokenize(grn_ctx *ctx);
+void grn_proc_init_tokenize(grn_ctx *ctx);
 
 grn_bool grn_proc_option_value_bool(grn_ctx *ctx,
                                     grn_obj *option,
@@ -77,15 +82,36 @@ const char *grn_proc_option_value_string(grn_ctx *ctx,
 void grn_proc_output_object_name(grn_ctx *ctx, grn_obj *obj);
 void grn_proc_output_object_id_name(grn_ctx *ctx, grn_id id);
 
-void grn_proc_table_set_token_filters(grn_ctx *ctx,
-                                      grn_obj *table,
-                                      grn_obj *token_filter_names);
+grn_bool grn_proc_table_set_token_filters(grn_ctx *ctx,
+                                          grn_obj *table,
+                                          grn_obj *token_filter_names);
 
-void grn_proc_select_output_columns(grn_ctx *ctx,
-                                    grn_obj *res,
-                                    int n_hits, int offset, int limit,
-                                    const char *columns, int columns_len,
-                                    grn_obj *condition);
+grn_column_flags grn_proc_column_parse_flags(grn_ctx *ctx,
+                                             const char *error_message_tag,
+                                             const char *text,
+                                             const char *end);
+
+grn_bool grn_proc_select_output_columns_open(grn_ctx *ctx,
+                                             grn_obj_format *format,
+                                             grn_obj *result_set,
+                                             int n_hits,
+                                             int offset,
+                                             int limit,
+                                             const char *columns,
+                                             int columns_len,
+                                             grn_obj *condition,
+                                             uint32_t n_additional_elements);
+grn_bool grn_proc_select_output_columns_close(grn_ctx *ctx,
+                                              grn_obj_format *format,
+                                              grn_obj *result_set);
+grn_bool grn_proc_select_output_columns(grn_ctx *ctx,
+                                        grn_obj *res,
+                                        int n_hits,
+                                        int offset,
+                                        int limit,
+                                        const char *columns,
+                                        int columns_len,
+                                        grn_obj *condition);
 
 grn_rc grn_proc_syntax_expand_query(grn_ctx *ctx,
                                     const char *query,
@@ -93,7 +119,13 @@ grn_rc grn_proc_syntax_expand_query(grn_ctx *ctx,
                                     grn_expr_flags flags,
                                     const char *query_expander_name,
                                     unsigned int query_expander_name_len,
-                                    grn_obj *expanded_query);
+                                    grn_obj *expanded_query,
+                                    const char *error_message_tag);
+
+grn_expr_flags grn_proc_expr_query_flags_parse(grn_ctx *ctx,
+                                               const char *query_flags,
+                                               size_t query_flags_size,
+                                               const char *error_message_tag);
 
 #ifdef __cplusplus
 }
