@@ -196,7 +196,7 @@ ctx_set_error_message(mrb_state *mrb, mrb_value self)
 
   mrb_get_args(mrb, "S", &error_message);
   grn_ctx_log(ctx, "%.*s",
-              RSTRING_LEN(error_message),
+              (int)RSTRING_LEN(error_message),
               RSTRING_PTR(error_message));
 
   return error_message;
@@ -258,7 +258,7 @@ grn_mrb_ctx_check(mrb_state *mrb)
   grn_ctx *ctx = (grn_ctx *)mrb->ud;
   grn_mrb_data *data = &(ctx->impl->mrb);
   struct RClass *module = data->module;
-  struct RClass *error_class;
+  struct RClass *error_class = NULL;
 #define MESSAGE_SIZE 4096
   char message[MESSAGE_SIZE];
 
@@ -705,12 +705,49 @@ grn_mrb_ctx_check(mrb_state *mrb)
                  "normalizer error: <%s>(%d)",
                  ctx->errbuf, ctx->rc);
     break;
-  default:
+  case GRN_TOKEN_FILTER_ERROR:
+    error_class = mrb_class_get_under(mrb, module, "TokenFilterError");
+    grn_snprintf(message, MESSAGE_SIZE, MESSAGE_SIZE,
+                 "token filter error: <%s>(%d)",
+                 ctx->errbuf, ctx->rc);
+    break;
+  case GRN_COMMAND_ERROR:
+    error_class = mrb_class_get_under(mrb, module, "CommandError");
+    grn_snprintf(message, MESSAGE_SIZE, MESSAGE_SIZE,
+                 "command error: <%s>(%d)",
+                 ctx->errbuf, ctx->rc);
+    break;
+  case GRN_PLUGIN_ERROR:
+    error_class = mrb_class_get_under(mrb, module, "PluginError");
+    grn_snprintf(message, MESSAGE_SIZE, MESSAGE_SIZE,
+                 "plugin error: <%s>(%d)",
+                 ctx->errbuf, ctx->rc);
+    break;
+  case GRN_SCORER_ERROR:
+    error_class = mrb_class_get_under(mrb, module, "ScorerError");
+    grn_snprintf(message, MESSAGE_SIZE, MESSAGE_SIZE,
+                 "scorer error: <%s>(%d)",
+                 ctx->errbuf, ctx->rc);
+    break;
+  case GRN_CANCEL:
+    error_class = mrb_class_get_under(mrb, module, "Cancel");
+    grn_snprintf(message, MESSAGE_SIZE, MESSAGE_SIZE,
+                 "cancel: <%s>(%d)",
+                 ctx->errbuf, ctx->rc);
+    break;
+  case GRN_WINDOW_FUNCTION_ERROR:
+    error_class = mrb_class_get_under(mrb, module, "WindowFunctionError");
+    grn_snprintf(message, MESSAGE_SIZE, MESSAGE_SIZE,
+                 "window function error: <%s>(%d)",
+                 ctx->errbuf, ctx->rc);
+    break;
+  }
+
+  if (!error_class) {
     error_class = mrb_class_get_under(mrb, module, "Error");
     grn_snprintf(message, MESSAGE_SIZE, MESSAGE_SIZE,
                  "unsupported error: <%s>(%d)",
                  ctx->errbuf, ctx->rc);
-    break;
   }
 #undef MESSAGE_SIZE
 
