@@ -20,15 +20,16 @@
 
 #include "../grn_ctx.h"
 #include "../grn_str.h"
+#include "../grn_db.h"
 
 #include <groonga/plugin.h>
 
-static grn_obj_flags
+static grn_table_flags
 command_table_create_parse_flags(grn_ctx *ctx,
                                  const char *nptr,
                                  const char *end)
 {
-  grn_obj_flags flags = 0;
+  grn_table_flags flags = 0;
   while (nptr < end) {
     size_t name_size;
 
@@ -118,10 +119,12 @@ grn_proc_table_set_token_filters_fill(grn_ctx *ctx,
       if (!name_end) {
         name_end = current;
       }
-      grn_proc_table_set_token_filters_put(ctx,
-                                           token_filters,
-                                           name_start,
-                                           name_end - name_start);
+      if (!grn_proc_table_set_token_filters_put(ctx,
+                                                token_filters,
+                                                name_start,
+                                                name_end - name_start)) {
+        return GRN_FALSE;
+      }
       last_name_end = name_end + 1;
       name_start = NULL;
       name_end = NULL;
@@ -158,24 +161,28 @@ break_loop:
   return GRN_TRUE;
 }
 
-void
+grn_bool
 grn_proc_table_set_token_filters(grn_ctx *ctx,
                                  grn_obj *table,
                                  grn_obj *token_filter_names)
 {
+  grn_bool succeeded = GRN_FALSE;
   grn_obj token_filters;
 
   if (GRN_TEXT_LEN(token_filter_names) == 0) {
-    return;
+    return GRN_TRUE;
   }
 
   GRN_PTR_INIT(&token_filters, GRN_OBJ_VECTOR, 0);
-  if (grn_proc_table_set_token_filters_fill(ctx,
-                                            &token_filters,
-                                            token_filter_names)) {
+  succeeded = grn_proc_table_set_token_filters_fill(ctx,
+                                                    &token_filters,
+                                                    token_filter_names);
+  if (succeeded) {
     grn_obj_set_info(ctx, table, GRN_INFO_TOKEN_FILTERS, &token_filters);
   }
   grn_obj_unlink(ctx, &token_filters);
+
+  return succeeded;
 }
 
 static grn_obj *
@@ -188,36 +195,46 @@ command_table_create(grn_ctx *ctx,
   grn_obj *flags_raw;
   grn_obj *key_type_name;
   grn_obj *value_type_name;
-  grn_obj *default_tokenizer;
-  grn_obj *normalizer;
-  grn_obj *token_filters;
+  grn_obj *default_tokenizer_name;
+  grn_obj *normalizer_name;
+  grn_obj *token_filters_name;
   grn_obj *table;
   const char *rest;
-  grn_obj_flags flags;
+  grn_table_flags flags;
 
   name = grn_plugin_proc_get_var(ctx, user_data, "name", -1);
   flags_raw = grn_plugin_proc_get_var(ctx, user_data, "flags", -1);
   key_type_name = grn_plugin_proc_get_var(ctx, user_data, "key_type", -1);
   value_type_name = grn_plugin_proc_get_var(ctx, user_data, "value_type", -1);
-  default_tokenizer =
+  default_tokenizer_name =
     grn_plugin_proc_get_var(ctx, user_data, "default_tokenizer", -1);
-  normalizer =
+  normalizer_name =
     grn_plugin_proc_get_var(ctx, user_data, "normalizer", -1);
-  token_filters =
+  token_filters_name =
     grn_plugin_proc_get_var(ctx, user_data, "token_filters", -1);
 
   flags = grn_atoi(GRN_TEXT_VALUE(flags_raw),
                    GRN_BULK_CURR(flags_raw),
                    &rest);
+
   if (GRN_TEXT_VALUE(flags_raw) == rest) {
     flags = command_table_create_parse_flags(ctx,
                                              GRN_TEXT_VALUE(flags_raw),
                                              GRN_BULK_CURR(flags_raw));
     if (ctx->rc) { goto exit; }
   }
-  if (GRN_TEXT_LEN(name)) {
+
+  if (GRN_TEXT_LEN(name) == 0) {
+    GRN_PLUGIN_ERROR(ctx,
+                     GRN_INVALID_ARGUMENT,
+                     "[table][create] should not create anonymous table");
+    goto exit;
+  }
+
+  {
     grn_obj *key_type = NULL;
     grn_obj *value_type = NULL;
+
     if (GRN_TEXT_LEN(key_type_name) > 0) {
       key_type = grn_ctx_get(ctx,
                              GRN_TEXT_VALUE(key_type_name),
@@ -231,9 +248,10 @@ command_table_create(grn_ctx *ctx,
                          GRN_TEXT_VALUE(name),
                          (int)GRN_TEXT_LEN(key_type_name),
                          GRN_TEXT_VALUE(key_type_name));
-        return NULL;
+        goto exit;
       }
     }
+
     if (GRN_TEXT_LEN(value_type_name) > 0) {
       value_type = grn_ctx_get(ctx,
                                GRN_TEXT_VALUE(value_type_name),
@@ -247,9 +265,10 @@ command_table_create(grn_ctx *ctx,
                          GRN_TEXT_VALUE(name),
                          (int)GRN_TEXT_LEN(value_type_name),
                          GRN_TEXT_VALUE(value_type_name));
-        return NULL;
+        goto exit;
       }
     }
+
     flags |= GRN_OBJ_PERSISTENT;
     table = grn_table_create(ctx,
                              GRN_TEXT_VALUE(name),
@@ -257,29 +276,64 @@ command_table_create(grn_ctx *ctx,
                              NULL, flags,
                              key_type,
                              value_type);
-    if (table) {
+    if (!table) {
+      goto exit;
+    }
+
+    if (GRN_TEXT_LEN(default_tokenizer_name) > 0) {
+      grn_obj *default_tokenizer;
+
+      default_tokenizer =
+        grn_ctx_get(ctx,
+                    GRN_TEXT_VALUE(default_tokenizer_name),
+                    GRN_TEXT_LEN(default_tokenizer_name));
+      if (!default_tokenizer) {
+        GRN_PLUGIN_ERROR(ctx,
+                         GRN_INVALID_ARGUMENT,
+                         "[table][create][%.*s] unknown tokenizer: <%.*s>",
+                         (int)GRN_TEXT_LEN(name),
+                         GRN_TEXT_VALUE(name),
+                         (int)GRN_TEXT_LEN(default_tokenizer_name),
+                         GRN_TEXT_VALUE(default_tokenizer_name));
+        grn_obj_remove(ctx, table);
+        goto exit;
+      }
       grn_obj_set_info(ctx, table,
                        GRN_INFO_DEFAULT_TOKENIZER,
-                       grn_ctx_get(ctx,
-                                   GRN_TEXT_VALUE(default_tokenizer),
-                                   GRN_TEXT_LEN(default_tokenizer)));
-      if (GRN_TEXT_LEN(normalizer) > 0) {
-        grn_obj_set_info(ctx, table,
-                         GRN_INFO_NORMALIZER,
-                         grn_ctx_get(ctx,
-                                     GRN_TEXT_VALUE(normalizer),
-                                     GRN_TEXT_LEN(normalizer)));
-      }
-      grn_proc_table_set_token_filters(ctx, table, token_filters);
-      grn_obj_unlink(ctx, table);
+                       default_tokenizer);
     }
-  } else {
-    GRN_PLUGIN_ERROR(ctx,
-                     GRN_INVALID_ARGUMENT,
-                     "[table][create] should not create anonymous table");
+
+    if (GRN_TEXT_LEN(normalizer_name) > 0) {
+      grn_obj *normalizer;
+
+      normalizer =
+        grn_ctx_get(ctx,
+                    GRN_TEXT_VALUE(normalizer_name),
+                    GRN_TEXT_LEN(normalizer_name));
+      if (!normalizer) {
+        GRN_PLUGIN_ERROR(ctx,
+                         GRN_INVALID_ARGUMENT,
+                         "[table][create][%.*s] unknown normalizer: <%.*s>",
+                         (int)GRN_TEXT_LEN(name),
+                         GRN_TEXT_VALUE(name),
+                         (int)GRN_TEXT_LEN(normalizer_name),
+                         GRN_TEXT_VALUE(normalizer_name));
+        grn_obj_remove(ctx, table);
+        goto exit;
+      }
+      grn_obj_set_info(ctx, table, GRN_INFO_NORMALIZER, normalizer);
+    }
+
+    if (!grn_proc_table_set_token_filters(ctx, table, token_filters_name)) {
+      grn_obj_remove(ctx, table);
+      goto exit;
+    }
+
+    grn_obj_unlink(ctx, table);
   }
+
 exit :
-  grn_ctx_output_bool(ctx, !ctx->rc);
+  grn_ctx_output_bool(ctx, ctx->rc == GRN_SUCCESS);
   return NULL;
 }
 
@@ -308,8 +362,10 @@ output_table_info(grn_ctx *ctx, grn_obj *table)
   grn_id id;
   grn_obj o;
   const char *path;
+  grn_table_flags flags;
   grn_obj *default_tokenizer;
   grn_obj *normalizer;
+  grn_obj *token_filters;
 
   id = grn_obj_id(ctx, table);
   path = grn_obj_path(ctx, table);
@@ -319,16 +375,19 @@ output_table_info(grn_ctx *ctx, grn_obj *table)
   grn_proc_output_object_id_name(ctx, id);
   grn_ctx_output_cstr(ctx, path);
   GRN_BULK_REWIND(&o);
-  grn_dump_table_create_flags(ctx, table->header.flags, &o);
+
+  grn_table_get_info(ctx, table,
+                     &flags,
+                     NULL,
+                     &default_tokenizer,
+                     &normalizer,
+                     &token_filters);
+  grn_dump_table_create_flags(ctx, flags, &o);
   grn_ctx_output_obj(ctx, &o, NULL);
   grn_proc_output_object_id_name(ctx, table->header.domain);
   grn_proc_output_object_id_name(ctx, grn_obj_get_range(ctx, table));
-  default_tokenizer = grn_obj_get_info(ctx, table, GRN_INFO_DEFAULT_TOKENIZER,
-                                       NULL);
   grn_proc_output_object_name(ctx, default_tokenizer);
-  normalizer = grn_obj_get_info(ctx, table, GRN_INFO_NORMALIZER, NULL);
   grn_proc_output_object_name(ctx, normalizer);
-  grn_obj_unlink(ctx, normalizer);
   grn_ctx_output_array_close(ctx);
   GRN_OBJ_FIN(ctx, &o);
   return 1;
